from .field import FormField, FormFieldData, FormFieldInfo, FormFilter
from .filters import FormFilter
from .form import Form, FormMeta
from .state import FormState

__all__ = (
    "FormField",
    "FormFieldInfo",
    "FormFilter",
    "Form",
    "FormMeta",
    "FormState",
)



import datetime
from typing import Any, Awaitable, Callable, Union

from aiogram import F, types
from aiogram.utils.magic_filter import MagicFilter

FormFilter = Union[MagicFilter, Callable[..., Awaitable[Any]]]

DEFAULT_FORM_FILTERS = {
    str: F.text,
    int: F.text.func(int),
    float: F.text.func(float),
    datetime.date: F.text.func(
        lambda text: datetime.datetime.strptime(text, r"%d.%m.%Y").date()
    ),
    datetime.datetime: F.text.func(
        lambda text: datetime.datetime.strptime(text, r"%d.%m.%Y %H:%M")
    ),
    datetime.time: F.text.func(
        lambda text: datetime.datetime.strptime(text, r"%H:%M").time()
    ),
    types.PhotoSize: F.photo.func(lambda photo: photo[-1]),
    types.Document: F.document.func(lambda document: document),
    types.Message: F.func(lambda m: m),
}



from dataclasses import dataclass
from typing import Any, Awaitable, Callable, Optional, Type, Union

from aiogram import types
from aiogram.utils.magic_filter import MagicFilter

Markup = Union[types.ReplyKeyboardMarkup, types.InlineKeyboardMarkup]
FormFilter = Union[MagicFilter, Callable[..., Awaitable[Any]]]

# TODO: pass di data into enter callback
EnterCallback = Callable[[int, int, "FormFieldData"], Awaitable[Any]]


@dataclass(frozen=True)
class FormFieldInfo:
    enter_message_text: Optional[str]
    error_message_text: Optional[str]
    filter: Optional[FormFilter]
    reply_markup: Optional[Markup]
    enter_callback: Optional[EnterCallback]


@dataclass
class FormFieldData:
    name: str
    type: Type
    info: FormFieldInfo


def FormField(
    *,
    enter_message_text: Optional[str] = None,
    filter: Optional[FormFilter] = None,
    error_message_text: Optional[str] = None,
    reply_markup: Optional[Markup] = None,
    enter_callback: Optional[EnterCallback] = None
) -> Any:
    if enter_message_text is None and enter_callback is None:
        raise ValueError("enter_message_text or enter_callback should be set")

    return FormFieldInfo(
        enter_message_text=enter_message_text,
        error_message_text=error_message_text,
        filter=filter,
        reply_markup=reply_markup,
        enter_callback=enter_callback,
    )
